#!/usr/bin/env perl

use Test2::V0;
use Test2::Require::AuthorTesting;

use Dev::Util::Syntax;

use Test2::Require::Module 'Test::Portability::Files';
use Test::Portability::Files;

options( test_one_dot => 0 );
run_tests();
